@@ -6,13 +6,14 @@
 -- exception handling via <code>nmap.new_try</code>.
 --
 -- These functions may be passed a table of options, but it's not required. The
--- keys for the options table are <code>"bytes"</code>, <code>"lines"</code>,
--- <code>"proto"</code>, and <code>"timeout"</code>. <code>"bytes"</code> sets
--- a minimum number of bytes to read. <code>"lines"</code> does the same for
--- lines. <code>"proto"</code> sets the protocol to communicate with,
--- defaulting to <code>"tcp"</code> if not provided. <code>"timeout"</code>
--- sets the socket timeout (see the socket function <code>set_timeout</code>
--- for details). 
+-- keys for the options table are:
+-- * <code>bytes</code> - minimum number of bytes to read.
+-- * <code>lines</code> - minimum number of lines to read.
+-- * <code>proto</code> - string, protocol to use. Default <code>"tcp"</code>
+-- * <code>timeout</code> - socket timeout in milliseconds. Default 8000
+-- * <code>connect_timeout</code> - override timeout for connection
+-- * <code>request_timeout</code> - override timeout for requests
+-- * <code>recv_before</code> - boolean, receive data before sending first payload
 --
 -- If both <code>"bytes"</code> and <code>"lines"</code> are provided,
 -- <code>"lines"</code> takes precedence. If neither are given, the functions
@@ -20,10 +21,10 @@
 -- @author Kris Katterjohn 04/2008
 -- @copyright Same as Nmap--See http://nmap.org/book/man-legal.html
 
-module(... or "comm", package.seeall)
-
-require 'nsedebug'
-require 'datafiles'
+local nmap = require "nmap"
+local shortport = require "shortport"
+local stdnse = require "stdnse"
+_ENV = stdnse.module("comm", stdnse.seeall)
 
 -- Makes sure that opts exists and the default proto is there
 local initopts = function(opts)
@@ -131,24 +132,17 @@ exchange = function(host, port, data, opts)
     return status, ret
 end
 
---- This function just checks if the provided port number is on a list
--- of ports that usually provide services with ssl
+--- This function uses shortport.ssl to check if the port is a likely SSL port
+-- @see shortport.ssl
 --
--- @param port_number The number of the port to check
+-- @param port The port table to check
 -- @return bool True if port is usually ssl, otherwise false
-local function is_ssl(port_number)
-    local common_ssl_ports = {443, 465, 989, 990, 992, 993, 994, 995, 587, 6697, 6679, 8443}
-    local table_size = table.maxn(common_ssl_ports) 
-    local i = 0
-    while i < table_size do
-        if port_number == common_ssl_ports[i] then return true end
-        i = i + 1
-    end
-    return false
+local function is_ssl(port)
+  return shortport.ssl(nil, port)
 end
 
---- This function returns best protocol order for trying  to open a 
--- conenction based on port and service information
+--- This function returns best protocol order for trying  to open a
+-- connection based on port and service information
 --
 -- The first value is the best option, the second is the worst
 -- @param port The port table
@@ -158,21 +152,21 @@ local function bestoption(port)
     if type(port) == 'table' then
         if port.version and port.version.service_tunnel and port.version.service_tunnel == "ssl" then return "ssl","tcp" end
         if port.version and port.version.name_confidence and port.version.name_confidence > 6 then return "tcp","ssl" end
-        if is_ssl(port.number) then return "ssl","tcp" end
-    elseif type(port) == 'number' then
         if is_ssl(port) then return "ssl","tcp" end
+    elseif type(port) == 'number' then
+      if is_ssl({number=port, protocol="tcp", state="open", version={}}) then return "ssl","tcp" end
     end
     return "tcp","ssl"
 end
 
 --- This function opens a connection, sends the first data payload and
---  check if a response is correctly received (what means that the 
+--  check if a response is correctly received (what means that the
 --  protocol used is fine)
 --
 -- Possible options:
 -- timeout: generic timeout value
--- connect_timeout: especific timeout for connection
--- request_timeout: especific timeout for requests
+-- connect_timeout: specific timeout for connection
+-- request_timeout: specific timeout for requests
 -- recv_before: receive data before sending first payload
 --
 -- Default timeout is set to 8000.
@@ -181,7 +175,7 @@ end
 -- @param port The destination host port
 -- @param protocol The protocol for the connection
 -- @param data The first data payload of the connection
--- @return sd The socket descriptor, nil if no connection is estabilished
+-- @return sd The socket descriptor, nil if no connection is established
 -- @return response The response received for the payload
 -- @return early_resp If opt recv_before is true, returns the value
 -- of the first receive (before sending data)
@@ -190,7 +184,7 @@ local function opencon(host, port, protocol, data, opts)
 
     -- check for connect_timeout or timeout option
 
-    if opts and opts.connect_timeout then 
+    if opts and opts.connect_timeout then
         sd:set_timeout(opts.connect_timeout)
     elseif opts and opts.timeout then
         sd:set_timeout(opts.timeout)
@@ -199,9 +193,9 @@ local function opencon(host, port, protocol, data, opts)
     end
 
     local status = sd:connect(host, port, protocol)
-    if not status then 
+    if not status then
           sd:close()
-          return nil, nil, nil 
+          return nil, nil, nil
         end
 
     -- check for request_timeout or timeout option
@@ -216,19 +210,19 @@ local function opencon(host, port, protocol, data, opts)
 
     local response, early_resp;
     if opts and opts.recv_before then status, early_resp = read(sd, opts) end
-    if #data > 0 then
+    if data and #data > 0 then
         sd:send(data)
         status, response = sd:receive()
     else
-        if not opts and opts.recv_before then
-            nsedebug.print_debug("Using comm.tryssl without first data payload and recv_first." .. 
+        if not (opts and opts.recv_before) then
+            stdnse.print_debug("Using comm.tryssl without either first data payload or opts.recv_before." ..
                          "\nImpossible to test the connection for the correct protocol!")
         end
         response = early_resp
     end
-    if not status then 
+    if not status then
           sd:close()
-          return nil, response, early_resp 
+          return nil, response, early_resp
         end
     return sd, response, early_resp
 end
@@ -262,3 +256,15 @@ function tryssl(host, port, data, opts)
     if not sd then best = "none" end
     return sd, response, best, early_resp
 end
+
+local unittest = require "unittest"
+if not unittest.testing() then
+  return _ENV
+end
+test_suite = unittest.TestSuite:new()
+test_suite:add_test(unittest.table_equal({bestoption(443)}, {"ssl", "tcp"}), "bestoption ssl number")
+test_suite:add_test(unittest.table_equal({bestoption(80)}, {"tcp", "ssl"}), "bestoption tcp number")
+test_suite:add_test(unittest.table_equal({bestoption({number=8443,protocol="tcp",state="open",version={}})}, {"ssl", "tcp"}), "bestoption ssl table")
+test_suite:add_test(unittest.table_equal({bestoption({number=1234,protocol="tcp",state="open",version={}})}, {"tcp", "ssl"}), "bestoption tcp table")
+
+return _ENV;
